from thd_json import Validator

from pathlib import Path
import argparse

import numpy as np
from thd_json.header import JsonHeader, generate_header
from thd_json.dict_converter import (
    position_to_dict,
    quaternion_to_dict,
)


def get_projection_geometry_validator(json_suffix: str = "*.json") -> Validator:
    return Validator(
        Path(__file__).parent / Path("projection_geometry.json"), json_suffix
    )


def projection_geometry_cli():
    parser = argparse.ArgumentParser(
        description="JSON THD Projection Geometry Validator CLI with uv."
    )
    parser.add_argument("folder", help="Folder to check.", type=str)
    parser.add_argument(
        "suffix", help="Projection suffix.", default="*.json", type=Path, nargs="?"
    )
    args = parser.parse_args()

    suffix = str(args.suffix)
    if not suffix.startswith("*"):
        raise ValueError(f'The suffix must always start with: "*". \nIt is: {suffix}')

    validator = get_projection_geometry_validator(suffix)

    folder = Path(args.folder)
    if not folder.exists():
        raise FileNotFoundError(f"Folder: {folder} does not exist")

    validator.folder(Path(args.folder))


def get_projection_geometry_dict(
    focal_spot_position_mm: np.ndarray,
    detector_center_position_mm: np.ndarray,
    detector_center_orientation_quat: np.ndarray,
    header: JsonHeader | None = None,
):
    if header is None:
        header = generate_header()

    projection_geometry = dict()
    projection_geometry["focal_spot_position_mm"] = position_to_dict(
        focal_spot_position_mm
    )
    projection_geometry["detector_center_position_mm"] = position_to_dict(
        detector_center_position_mm
    )
    projection_geometry["detector_center_orientation_quat"] = quaternion_to_dict(
        detector_center_orientation_quat
    )

    projection_geometry["header"] = {"uuid": header.uuid}

    return projection_geometry


if __name__ == "__main__":
    projection_geometry_cli()
